/**
 * Copyright (c) 2021 Jelle Hellings.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY JELLE HELLINGS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <iostream>
#include <string>
#include <vector>


/**
 * This data plot tool reads the raw measurements of the nine ByShard
 * experiments from standard input and writes them out in a PGFPlots friendly
 * format to standard output.
 *
 * The input (the raw measurements) will contain a single table per experiment
 * containing measurements for all protocols involved in that experiment. The
 * output will contain, for each experiment and protocol, a separate table.
 */

std::vector<std::string> headers = {"experiment_scalability",
                                    "experiment_contention",
                                    "experiment_factor_scalability"};

std::vector<std::string> data_names = {"Scalability",
                                       "Contention",
                                       "FactorScalability"};

std::vector<std::string> protocol_names = {"LOUS", "LOS", "LRU", "LRUWF", "LRC", "LRCWF", "LS", "LSWF",
                                           "COUS", "COS", "CRUWF", "CRCWF", "CSWF",
                                           "DOUS", "DOS", "DRUWF", "DRCWF", "DSWF",
                                           "AHL"};
void do_data(const std::string& data_name)
{
    std::string header_line;
    std::getline(std::cin, header_line);
    std::vector<std::vector<std::string>> regroup_lines(protocol_names.size());

    std::string line;
    auto it = regroup_lines.begin();
    while (std::getline(std::cin, line) && !line.empty()) {
        it->emplace_back(line);
        ++it;
        if (it == regroup_lines.end()) {
            it = regroup_lines.begin();
        }
    }

    auto protocol_name_it = protocol_names.cbegin();
    for (const auto& lines : regroup_lines) {
        std::cout  << "\\pgfplotstableread{" << std::endl
                   << header_line << std::endl;

        for (const auto& line : lines) {
            std::cout << line << std::endl;
        }

        std::cout << "}\\data" << data_name << *protocol_name_it << std::endl << std::endl;
        ++protocol_name_it;
    }
}

int main(int argc, char* argv[])
{
    std::string line;

    auto header_it = headers.cbegin();
    auto data_name_it = data_names.cbegin();
    while (std::getline(std::cin, line) && (header_it != headers.cend())) {
        if (line == *header_it) {
            do_data(*data_name_it);
        }
        ++header_it;
        ++data_name_it;
    }
}